import { Injectable } from '@angular/core';
import { invoke } from '@tauri-apps/api';
import { Observable, ReplaySubject, Subject, from, map, tap } from 'rxjs';
import { Plan, PlanInterface } from '../_models/plan';

@Injectable({
  providedIn: 'root'
})
export class PlanService {

  currentPlan?: Plan;
  planStore: string[] = [];
  basePlan?: Plan;
  private basePlanSubj: Subject<Plan> = new ReplaySubject<Plan>(1);

  

  constructor() {
    this.getPreviousPlans();
    this.loadBasePlan();
  }

  loadPlan(path: string) {
    return from(invoke<PlanInterface>('load_plan', { path })).pipe(
      map(plan => {
        this.currentPlan = new Plan(plan);
        return plan
      })
    );
  }

  loadPlanNoSave(path: string) {
    return from(invoke<PlanInterface>('load_plan', { path })).pipe(
      map(plan => {
        this.currentPlan = new Plan(plan);
        this.currentPlan.setPath(path);
        return plan
      })
    );
  }

  

  loadBasePlan() {
    if(!this.basePlan) {
      from(invoke<PlanInterface>('base_plan')).subscribe(plan => {
        plan.plan.forEach(elem => {elem.edited = false;});
        this.basePlan = new Plan(plan);
        this.basePlanSubj?.next(this.basePlan);
      });
    }

    return this.basePlanSubj!.asObservable();
  }

  savePlan(path: string, plan: Plan) {
    plan.plan.forEach(elem => {
      if (!elem.notes) { elem.notes = "" }
    });

    return from(invoke<boolean>('save_plan', {
      path,
      plan: {
        plan: plan.plan,
        current: plan.current
      },
    })).subscribe(status => {
    });
  }

  getPreviousPlans() {
    from(invoke<string[]>('load_stored_plans')).subscribe(plans => this.planStore = plans);
  }

  loadPrevious(name: string) {
    return from(invoke<PlanInterface>('load_previous', { name })).pipe(tap(plan => {
      console.log("previous loaded: ", plan);
      this.currentPlan = new Plan(plan);
      this.currentPlan.setPrevious(name);
    }));
  }

  zoneFromUuid(uuid: string) {
    if (!this.basePlan) {
      return undefined;
    }

    return this.basePlan.plan.find(elem => elem.uuid === uuid);
  }
}
